unit PersianDate;

//----------------------------------------------------------------------------//
//                                                                            //
//                           Persian Date 1.0.1                               //
//                          By Mohammad Yousefi                               //
//                        vahid_you2004@yahoo.com                             //
//                               April 2013                                   //
//                         http://www.dihav.com/                              //
//                                                                            //
//----------------------------------------------------------------------------//

interface

uses SysUtils, Windows, SysConst;

type
  TDateOrder=(doYMD,doYDM,doMDY,doMYD,doDMY,doDYM);

//The functions are the same as delphi date functions.
//Note that in TSystemTime wDayOfWeek specifies the day of the week starting from zero; Saturday = 0, Sunday = 1, and so on.
//Note that in other functions DayOfWeek specifies the day of the week starting from one; Saturday = 1, Sunday = 2, and so on.

  //DateTime Types Conversions
  function PersianSystemTimeToTimeStamp(PersianSystemTime:TSystemTime):TTimeStamp;
  function PersianTimeStampToSystemTime(PersianTimeStamp:TTimeStamp):TSystemTime;
  function PersianTimeStampToDateTime(PersianTimeStamp:TTimeStamp):TDateTime;
  function DateTimeToPersianTimeStamp(DateTime:TDateTime):TTimeStamp;
  function PersianSystemTimeToDateTime(PersianSystemTime:TSystemTime):TDateTime;
  function DateTimeToPersianSystemTime(DateTime:TDateTime):TSystemTime;

  //Persian/Gregorian Date Conversions
  function GregorianTimeStampToPersianTimeStamp(GregorianTimeStamp:TTimeStamp):TTimeStamp;
  function PersianTimeStampToGregorianTimeStamp(PersianTimeStamp:TTimeStamp):TTimeStamp;
  function GregorianSystemTimeToPersianSystemTime(GregorianSystemTime:TSystemTime):TSystemTime;
  function PersianSystemTimeToGregorianSystemTime(PersianSystemTime:TSystemTime):TSystemTime;
  function GregorianDateTimeToPersianTimeStamp(GregorianDateTime:TDateTime):TTimeStamp;
  function PersianTimeStampToGregorianDateTime(PersianTimeStamp:TTimeStamp):TDateTime;
  function GregorianDateTimeToPersianSystemTime(GregorianDateTime:TDateTime):TSystemTime;
  function PersianSystemTimeToGregorianDateTime(PersianSystemTime:TSystemTime):TDateTime;


  //Encode/Decode
  function MakeSystemTime(wYear,wMonth,wDay,wDayOfWeek,wHour,wMinute,wSecond,wMilliseconds:Word):TSystemTime;
  function EncodePersianDate(Year,Month,Day:Word):TDateTime;
  function TryEncodePersianDate(Year,Month,Day:Word;out Date:TDateTime):Boolean;
  procedure DecodePersianDate(const DateTime:TDateTime;var Year,Month,Day:Word);
  function DecodePersianDateFully(const DateTime:TDateTime;var Year,Month,Day,DOW:Word):Boolean;

  //Persian Date
  function IsPersianLeapYear(Year:Integer):Boolean;
  function ThePersianYear(aDate:TDateTime):Word;
  function ThePersianMonth(aDate:TDateTime):Word;
  function ThePersianMonthLongName(aDate:TDateTime;const FormatSettings:TFormatSettings):String; overload;
  function ThePersianMonthLongName(aDate:TDateTime):String; overload;
  function ThePersianMonthShortName(aDate:TDateTime;const FormatSettings:TFormatSettings):String; overload;
  function ThePersianMonthShortName(aDate:TDateTime):String; overload;
  function PersianDayOfTheMonth(aDate:TDateTime):Word;
  function PersianDayOfTheYear(aDate:TDateTime):Word;
  function PersianDayOfTheWeek(aDate:TDateTime):Word;
  function PersianDayOfTheWeekLongName(aDate:TDateTime;const FormatSettings:TFormatSettings):String; overload;
  function PersianDayOfTheWeekLongName(aDate:TDateTime):String; overload;
  function PersianDayOfTheWeekShortName(aDate:TDateTime;const FormatSettings:TFormatSettings):String; overload;
  function PersianDayOfTheWeekShortName(aDate:TDateTime):String; overload;
  function CurrentPersianYear:Word;
  function CurrentPersianMonth:Word;
  function CurrentPersianMonthLongName(const FormatSettings:TFormatSettings):String; overload;
  function CurrentPersianMonthLongName:String; overload;
  function CurrentPersianMonthShortName(const FormatSettings:TFormatSettings):String; overload;
  function CurrentPersianMonthShortName:String; overload;
  function CurrentPersianDayOfMonth:Word;
  function CurrentPersianDayOfYear:Word;
  function CurrentPersianDayOfWeek:Word;
  function CurrentPersianDayOfWeekLongName(const FormatSettings:TFormatSettings):String; overload;
  function CurrentPersianDayOfWeekLongName:String; overload;
  function CurrentPersianDayOfWeekShortName(const FormatSettings:TFormatSettings):String; overload;
  function CurrentPersianDayOfWeekShortName:String; overload;

  //Date Shift
  function IncPersianDay(const DateTime:TDateTime;NumberOfDays:Integer=1):TDateTime;
  procedure IncAPersianDay(var Year,Month,Day:Word;NumberOfDays:Integer=1);
  function DecPersianDay(const DateTime:TDateTime;NumberOfDays:Integer=1):TDateTime;
  procedure DecAPersianDay(var Year,Month,Day:Word;NumberOfDays:Integer=1);
  function IncPersianWeek(const DateTime:TDateTime;NumberOfWeeks:Integer=1):TDateTime;
  procedure IncAPersianWeek(var Year,Month,Day:Word;NumberOfWeeks:Integer=1);
  function DecPersianWeek(const DateTime:TDateTime;NumberOfWeeks:Integer=1):TDateTime;
  procedure DecAPersianWeek(var Year,Month,Day:Word;NumberOfWeeks:Integer=1);
  function IncPersianMonth(const DateTime:TDateTime;NumberOfMonths:Integer=1):TDateTime;
  procedure IncAPersianMonth(var Year,Month,Day:Word;NumberOfMonths:Integer=1);
  function DecPersianMonth(const DateTime:TDateTime;NumberOfMonths:Integer=1):TDateTime;
  procedure DecAPersianMonth(var Year,Month,Day:Word;NumberOfMonths:Integer=1);
  function IncPersianYear(const DateTime:TDateTime;NumberOfYears:Integer=1):TDateTime;
  procedure IncAPersianYear(var Year,Month,Day:Word;NumberOfYears:Integer=1);
  function DecPersianYear(const DateTime:TDateTime;NumberOfYears:Integer=1):TDateTime;
  procedure DecAPersianYear(var Year,Month,Day:Word;NumberOfYears:Integer=1);

  //Conversions to String
  procedure PersianSystemTimeToString(var Result:string;const Format:String;PersianSystemTime:TSystemTime;const FormatSettings:TFormatSettings); overload;
  procedure PersianSystemTimeToString(var Result:string;const Format:String;PersianSystemTime:TSystemTime); overload;
  procedure PersianTimeStampToString(var Result:string;const Format:String;PersianTimeStamp:TTimeStamp;const FormatSettings:TFormatSettings); overload;
  procedure PersianTimeStampToString(var Result:string;const Format:String;PersianTimeStamp:TTimeStamp); overload;
  procedure PersianDateTimeToString(var Result:string;const Format:String;DateTime:TDateTime;const FormatSettings:TFormatSettings); overload;
  procedure PersianDateTimeToString(var Result:string;const Format:String;DateTime:TDateTime); overload;
  function PersianDateToStr(const DateTime:TDateTime;const FormatSettings:TFormatSettings):string; overload;
  function PersianDateToStr(const DateTime:TDateTime):string; overload;
  function PersianDateTimeToStr(const DateTime:TDateTime;const FormatSettings:TFormatSettings):string; overload;
  function PersianDateTimeToStr(const DateTime:TDateTime):string; overload;
  function FormatPersianDateTime(const Format:string;DateTime:TDateTime;const FormatSettings:TFormatSettings):string; overload;
  function FormatPersianDateTime(const Format:string;DateTime:TDateTime):string; overload;

  //Read from String
  function StrToPersianDate(const S:string):TDateTime; overload;
  function StrToPersianDate(const S:string;const FormatSettings:TFormatSettings):TDateTime; overload;
  function StrToPersianDateDef(const S:string;const Default:TDateTime):TDateTime; overload;
  function StrToPersianDateDef(const S:string;const Default:TDateTime;const FormatSettings:TFormatSettings):TDateTime; overload;
  function TryStrToPersianDate(const S:string;out Value:TDateTime):Boolean; overload;
  function TryStrToPersianDate(const S:string;out Value:TDateTime;const FormatSettings:TFormatSettings):Boolean; overload;
  function StrToPersianDateTime(const S:string):TDateTime; overload;
  function StrToPersianDateTime(const S:string;const FormatSettings:TFormatSettings):TDateTime; overload;
  function StrToPersianDateTimeDef(const S:string;const Default:TDateTime):TDateTime; overload;
  function StrToPersianDateTimeDef(const S:string;const Default:TDateTime;const FormatSettings:TFormatSettings):TDateTime; overload;
  function TryStrToPersianDateTime(const S:string;out Value:TDateTime):Boolean; overload;
  function TryStrToPersianDateTime(const S:string;out Value:TDateTime;const FormatSettings:TFormatSettings):Boolean; overload;
  function ExtractPersianDateFromString(const S:string;DateOrder:TDateOrder;out Value:TDateTime):Boolean; overload;
  function ExtractPersianDateFromString(const S:string;DateOrder:TDateOrder;out Value:TDateTime;const FormatSettings:TFormatSettings):Boolean; overload;

  //Numbers to Text
  function NumberToPersianText(I:Int64):String;
  function NumberToPersianOrdinalText(I:Int64):String;
  function NumberToText(I:Int64):String;
  function NumberToOrdinalText(I:Int64):String;
  function NumberToOrdinalNumber(I:Int64):String;
  

{ FormatDateTime formats the date-and-time value given by DateTime using the
  format given by Format. The following format specifiers are supported:

  c       Displays the date using the format given by the ShortDateFormat
          global variable, followed by the time using the format given by
          the LongTimeFormat global variable. The time is not displayed if
          the fractional part of the DateTime value is zero.

  d       Displays the day as a number without a leading zero (1-31).

  dd      Displays the day as a number with a leading zero (01-31).

  ddd     Displays the day as an abbreviation (Sun-Sat) using the strings
          given by the ShortDayNames global variable.

  dddd    Displays the day as a full name (Sunday-Saturday) using the strings
          given by the LongDayNames global variable.

  ddddd   Displays the date using the format given by the ShortDateFormat
          global variable.

  dddddd  Displays the date using the format given by the LongDateFormat
          global variable.

  m       Displays the month as a number without a leading zero (1-12). If
          the m specifier immediately follows an h or hh specifier, the
          minute rather than the month is displayed.

  mm      Displays the month as a number with a leading zero (01-12). If
          the mm specifier immediately follows an h or hh specifier, the
          minute rather than the month is displayed.

  mmm     Displays the month as an abbreviation (Jan-Dec) using the strings
          given by the ShortMonthNames global variable.

  mmmm    Displays the month as a full name (January-December) using the
          strings given by the LongMonthNames global variable.

  yy      Displays the year as a two-digit number (00-99).

  yyyy    Displays the year as a four-digit number (0000-9999).

  h       Displays the hour without a leading zero (0-23).

  hh      Displays the hour with a leading zero (00-23).

  n       Displays the minute without a leading zero (0-59).

  nn      Displays the minute with a leading zero (00-59).

  s       Displays the second without a leading zero (0-59).

  ss      Displays the second with a leading zero (00-59).

  z       Displays the millisecond without a leading zero (0-999).

  zzz     Displays the millisecond with a leading zero (000-999).

  t       Displays the time using the format given by the ShortTimeFormat
          global variable.

  tt      Displays the time using the format given by the LongTimeFormat
          global variable.

  am/pm   Uses the 12-hour clock for the preceding h or hh specifier, and
          displays 'am' for any hour before noon, and 'pm' for any hour
          after noon. The am/pm specifier can use lower, upper, or mixed
          case, and the result is displayed accordingly.

  a/p     Uses the 12-hour clock for the preceding h or hh specifier, and
          displays 'a' for any hour before noon, and 'p' for any hour after
          noon. The a/p specifier can use lower, upper, or mixed case, and
          the result is displayed accordingly.

  ampm    Uses the 12-hour clock for the preceding h or hh specifier, and
          displays the contents of the TimeAMString global variable for any
          hour before noon, and the contents of the TimePMString global
          variable for any hour after noon.

  /       Displays the date separator character given by the DateSeparator
          global variable.

  :       Displays the time separator character given by the TimeSeparator
          global variable.

  'xx'    Characters enclosed in single or double quotes are displayed as-is,
  "xx"    and do not affect formatting.

  Format specifiers may be written in upper case as well as in lower case
  letters--both produce the same result.

  If the string given by the Format parameter is empty, the date and time
  value is formatted as if a 'c' format specifier had been given.

  The following example:

    S := FormatDateTime('"The meeting is on" dddd, mmmm d, yyyy, ' +
      '"at" hh:mm AM/PM', StrToDateTime('2/15/95 10:30am'));

  assigns 'The meeting is on Wednesday, February 15, 1995 at 10:30 AM' to
  the string variable S. }

const
  PersianAndGregorianTimeStampDateDifference=226895;
  PersianDateDelta=DateDelta-PersianAndGregorianTimeStampDateDifference;

 {$if CompilerVersion > 21}
  PersianFormatSettings:TFormatSettings=(CurrencyString:'';
                                         CurrencyFormat:2;
                                         CurrencyDecimals:2;
                                         DateSeparator:'/';
                                         TimeSeparator:':';
                                         ListSeparator:'';
                                         ShortDateFormat:'yyyy/MM/dd';
                                         LongDateFormat:'yyyy/MM/dd';
                                         TimeAMString:'.';
                                         TimePMString:'.';
                                         ShortTimeFormat:'hh:mm AMPM';
                                         LongTimeFormat:'hh:mm:ss AMPM';
                                         ShortMonthNames:('','','','','','','','','','','','');
                                         LongMonthNames:('','','','','','','','','','','','');
                                         ShortDayNames:('','','','','','','');
                                         LongDayNames:('','ߝ','','坝','','̝','');
                                         ThousandSeparator:',';
                                         DecimalSeparator:'/';
                                         TwoDigitYearCenturyWindow:2;
                                         NegCurrFormat:3;);
  EnglishFormatSettings:TFormatSettings=(CurrencyString:'$';
                                         CurrencyFormat:2;
                                         CurrencyDecimals:2;
                                         DateSeparator:'/';
                                         TimeSeparator:':';
                                         ListSeparator:';';
                                         ShortDateFormat:'yyyy/MM/dd';
                                         LongDateFormat:'yyyy/MM/dd';
                                         TimeAMString:'AM';
                                         TimePMString:'PM';
                                         ShortTimeFormat:'hh:mm AMPM';
                                         LongTimeFormat:'hh:mm:ss AMPM';
                                         ShortMonthNames:('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
                                         LongMonthNames:('January','February','March','April','May','June','July','August','September','October','November','December');
                                         ShortDayNames:('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
                                         LongDayNames:('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday');
                                         ThousandSeparator:',';
                                         DecimalSeparator:'/';
                                         TwoDigitYearCenturyWindow:2;
                                         NegCurrFormat:3;);
 {$else}
  PersianFormatSettings:TFormatSettings=(CurrencyFormat:2;
                                         NegCurrFormat:3;
                                         ThousandSeparator:',';
                                         DecimalSeparator:'/';
                                         CurrencyDecimals:2;
                                         DateSeparator:'/';
                                         TimeSeparator:':';
                                         ListSeparator:'';
                                         CurrencyString:'';
                                         ShortDateFormat:'yyyy/MM/dd';
                                         LongDateFormat:'yyyy/MM/dd';
                                         TimeAMString:'.';
                                         TimePMString:'.';
                                         ShortTimeFormat:'hh:mm AMPM';
                                         LongTimeFormat:'hh:mm:ss AMPM';
                                         ShortMonthNames:('','','','','','','','','','','','');
                                         LongMonthNames:('','','','','','','','','','','','');
                                         ShortDayNames:('','','','','','','');
                                         LongDayNames:('','ߝ','','坝','','̝','');
                                         TwoDigitYearCenturyWindow:2;);
  EnglishFormatSettings:TFormatSettings=(CurrencyFormat:2;
                                         NegCurrFormat:3;
                                         ThousandSeparator:',';
                                         DecimalSeparator:'/';
                                         CurrencyDecimals:2;
                                         DateSeparator:'/';
                                         TimeSeparator:':';
                                         ListSeparator:';';
                                         CurrencyString:'$';
                                         ShortDateFormat:'yyyy/MM/dd';
                                         LongDateFormat:'yyyy/MM/dd';
                                         TimeAMString:'AM';
                                         TimePMString:'PM';
                                         ShortTimeFormat:'hh:mm AMPM';
                                         LongTimeFormat:'hh:mm:ss AMPM';
                                         ShortMonthNames:('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
                                         LongMonthNames:('January','February','March','April','May','June','July','August','September','October','November','December');
                                         ShortDayNames:('Sun','Mon','Tue','Wed','Thu','Fri','Sat');
                                         LongDayNames:('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday');
                                         TwoDigitYearCenturyWindow:2;);
 {$ifend}

  PersianDaySaturday=1;
  PersianDaySunday=2;
  PersianDayMonday=3;
  PersianDayTuesday=4;
  PersianDayWednesday=5;
  PersianDayThursday=6;
  PersianDayFriday=7;

  PersianMonthDays:array [Boolean] of TDayTable=
    ((31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 29),
     (31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 30));

//----------------------------------------------------------------------------//

implementation

procedure ConvertError(ResString:PResStringRec); local;
begin
raise EConvertError.CreateRes(ResString);
end;

procedure ConvertErrorFmt(ResString:PResStringRec;const Args:array of const); local;
begin
raise EConvertError.CreateResFmt(ResString,Args);
end;

function MakeSystemTime(wYear,wMonth,wDay,wDayOfWeek,wHour,wMinute,wSecond,wMilliseconds:Word):TSystemTime;
begin
Result.wYear:=wYear;
Result.wMonth:=wMonth;
Result.wDay:=wDay;
Result.wDayOfWeek:=wDayOfWeek;
Result.wHour:=wHour;
Result.wMinute:=wMinute;
Result.wSecond:=wSecond;
Result.wMilliseconds:=wMilliseconds;
end;

function IsPersianLeapYear(Year:Integer):Boolean;
begin
Result:=((Year*31+1178) mod 128)<=30;
end;

function PersianSystemTimeToTimeStamp(PersianSystemTime:TSystemTime):TTimeStamp;
var d,k,i:Integer;
begin
d:=0;
if (PersianSystemTime.wYear=0) or (PersianSystemTime.wMonth=0) or (PersianSystemTime.wDay=0) then d:=1;
if (PersianSystemTime.wMonth>12) or (PersianSystemTime.wDay>31) then d:=1;
if (PersianSystemTime.wMonth>6) and (PersianSystemTime.wDay>30) then d:=1;
if (PersianSystemTime.wMonth=12) and (PersianSystemTime.wDay=30) and (not IsPersianLeapYear(PersianSystemTime.wYear)) then d:=1;
if d=1 then ConvertError(@SDateEncodeError);

if (PersianSystemTime.wHour>23) or (PersianSystemTime.wMinute>59) or (PersianSystemTime.wSecond>59) or (PersianSystemTime.wMilliseconds>999) then d:=2;
if d=2 then ConvertError(@STimeEncodeError);

i:=1;
d:=0;
if PersianSystemTime.wYear>1300 then
  begin
  i:=1301;
  d:=315;
  end;
for k:=i to PersianSystemTime.wYear-1 do
  if IsPersianLeapYear(k) then
    Inc(d);
d:=(PersianSystemTime.wYear-1)*365+d+(PersianSystemTime.wMonth-1)*31+PersianSystemTime.wDay;
if PersianSystemTime.wMonth>7 then Dec(d,PersianSystemTime.wMonth-7);
Result.Date:=d;

Result.Time:=PersianSystemTime.wHour*3600000+PersianSystemTime.wMinute*60000+PersianSystemTime.wSecond*1000+PersianSystemTime.wMilliseconds;
end;

function PersianTimeStampToSystemTime(PersianTimeStamp:TTimeStamp):TSystemTime;
var d:Integer;
begin
while PersianTimeStamp.Time>86399999 do Dec(PersianTimeStamp.Time,86400000);
if (PersianTimeStamp.Date<1) or (PersianTimeStamp.Time<0) then
  ConvertErrorFmt(@SInvalidTimeStamp,[PersianTimeStamp.Date,PersianTimeStamp.Time]);

d:=PersianTimeStamp.Date;
Result.wYear:=1;
while d>=366 do
  begin
  Dec(d,365);
  if IsPersianLeapYear(Result.wYear) then Dec(d);
  Inc(Result.wYear);
  end;
if d=0 then Dec(Result.wYear);
if (d=365) and (not IsPersianLeapYear(Result.wYear)) then d:=0;
Dec(d);
if d=-1 then
  begin
  Result.wMonth:=12;
  Result.wDay:=29;
  if IsPersianLeapYear(Result.wYear) then Inc(Result.wDay);
  end
else if d>186 then
  begin
  Result.wMonth:=(d-186) div 30+7;
  Result.wDay:=(d-186) mod 30+1;
  end
else
  begin
  Result.wMonth:=d div 31+1;
  Result.wDay:=d mod 31+1;
  end;
Result.wDayOfWeek:=(PersianTimeStamp.Date+5) mod 7;

Result.wHour:=PersianTimeStamp.Time div 3600000;
d:=PersianTimeStamp.Time mod 3600000;
Result.wMinute:=d div 60000;
Result.wSecond:=(d mod 60000) div 1000;
Result.wMilliseconds:=PersianTimeStamp.Time mod 1000;
end;

function GregorianTimeStampToPersianTimeStamp(GregorianTimeStamp:TTimeStamp):TTimeStamp;
begin
Result.Time:=GregorianTimeStamp.Time;
Result.Date:=GregorianTimeStamp.Date-PersianAndGregorianTimeStampDateDifference;
end;

function PersianTimeStampToGregorianTimeStamp(PersianTimeStamp:TTimeStamp):TTimeStamp;
begin
Result.Time:=PersianTimeStamp.Time;
Result.Date:=PersianTimeStamp.Date+PersianAndGregorianTimeStampDateDifference;
end;

function GregorianDateTimeToPersianTimeStamp(GregorianDateTime:TDateTime):TTimeStamp;
begin
Result:=GregorianTimeStampToPersianTimeStamp(DateTimeToTimeStamp(GregorianDateTime));
end;

function PersianTimeStampToGregorianDateTime(PersianTimeStamp:TTimeStamp):TDateTime;
begin
Result:=TimeStampToDateTime(PersianTimeStampToGregorianTimeStamp(PersianTimeStamp));
end;

function GregorianDateTimeToPersianSystemTime(GregorianDateTime:TDateTime):TSystemTime;
begin
Result:=PersianTimeStampToSystemTime(GregorianDateTimeToPersianTimeStamp(GregorianDateTime));
end;

function PersianSystemTimeToGregorianDateTime(PersianSystemTime:TSystemTime):TDateTime;
begin
Result:=PersianTimeStampToGregorianDateTime(PersianSystemTimeToTimeStamp(PersianSystemTime));
end;

function GregorianSystemTimeToPersianSystemTime(GregorianSystemTime:TSystemTime):TSystemTime;
begin
Result:=GregorianDateTimeToPersianSystemTime(SystemTimeToDateTime(GregorianSystemTime));
end;

function PersianSystemTimeToGregorianSystemTime(PersianSystemTime:TSystemTime):TSystemTime;
begin
DateTimeToSystemTime(PersianSystemTimeToGregorianDateTime(PersianSystemTime),Result);
end;

function PersianTimeStampToDateTime(PersianTimeStamp:TTimeStamp):TDateTime;
begin
Result:=PersianTimeStamp.Date-PersianDateDelta;
if Result<0 then
  Result:=Result-PersianTimeStamp.Date/MSecsPerDay
else
  Result:=Result+PersianTimeStamp.Date/MSecsPerDay;
end;

function DateTimeToPersianTimeStamp(DateTime:TDateTime):TTimeStamp;
begin
Result.Date:=Round(Int(DateTime)+PersianDateDelta);
Result.Time:=Abs(Round(Frac(DateTime)*MSecsPerDay));
end;

function PersianSystemTimeToDateTime(PersianSystemTime:TSystemTime):TDateTime;
begin
Result:=PersianTimeStampToDateTime(PersianSystemTimeToTimeStamp(PersianSystemTime));
end;

function DateTimeToPersianSystemTime(DateTime:TDateTime):TSystemTime;
begin
Result:=PersianTimeStampToSystemTime(DateTimeToPersianTimeStamp(DateTime));
end;

procedure PersianSystemTimeToString(var Result:string;const Format:String;PersianSystemTime:TSystemTime;const FormatSettings:TFormatSettings);
var
  k,l,n:Integer;
  b,t,d:Boolean;
  f:String;

  procedure AddRes(Str:String);
  begin
  Result:=Result+Str;
  end;

  function FChar(Index:Integer):Char;
  begin
  if Index<=Length(f) then Result:=f[Index] else Result:=#0;
  end;

  function IsChar(Index:Integer;Chr:Char):Boolean;
  begin
  Result:=LowerCase(FChar(Index))=LowerCase(Chr);
  end;

  function Count(Chr:Char):Word;
  begin
  Result:=0;
  while IsChar(k,Chr) do
    begin
    Inc(Result);
    Inc(k);
    end;
  end;

begin
k:=1;
Result:='';
b:=False;
t:=False;
l:=-1;
d:=False;
f:=Format;
if f='' then f:='c';
while k<=Length(f) do
  begin
  if (FChar(k)='''') or (FChar(k)='"') then
    begin
    Inc(k);
    b:=not b;
    end;
  if b then
    begin
    AddRes(FChar(k));
    Inc(k);
    end
  else
    begin
    if IsChar(k,'c') then
      begin
      Count('c');
      Delete(f,k,1);
      if (PersianSystemTime.wHour<>0) and (PersianSystemTime.wMinute<>0) and (PersianSystemTime.wSecond<>0) then
        begin
        Insert(FormatSettings.LongTimeFormat,f,k);
        Insert(' ',f,k);
        end;
      Insert(FormatSettings.ShortDateFormat,f,k);
      end;
    case FChar(k) of
      'd','D':
        begin
        case Count('d') of
          1:AddRes(SysUtils.Format('%d',[PersianSystemTime.wDay]));
          2:AddRes(SysUtils.Format('%.2d',[PersianSystemTime.wDay]));
          3:AddRes(FormatSettings.ShortDayNames[PersianSystemTime.wDayOfWeek+1]);
          4:AddRes(FormatSettings.LongDayNames[PersianSystemTime.wDayOfWeek+1]);
          5:Insert(FormatSettings.ShortDateFormat,f,k);
        else
          Insert(FormatSettings.LongDateFormat,f,k);
        end;
        t:=False;
        end;

      'm','M':
        begin
        case Count('m') of
          1:if t then AddRes(SysUtils.Format('%d',[PersianSystemTime.wMinute])) else AddRes(SysUtils.Format('%d',[PersianSystemTime.wMonth]));
          2:if t then AddRes(SysUtils.Format('%.2d',[PersianSystemTime.wMinute])) else AddRes(SysUtils.Format('%.2d',[PersianSystemTime.wMonth]));
          3:AddRes(FormatSettings.ShortMonthNames[PersianSystemTime.wMonth]);
        else
          AddRes(FormatSettings.LongMonthNames[PersianSystemTime.wMonth]);
        end;
        t:=False;
        end;

      'y','Y':
        begin
        if Count('y')<3 then
          AddRes(SysUtils.Format('%.2d',[PersianSystemTime.wYear mod 100]))
        else
          AddRes(SysUtils.Format('%.4d',[PersianSystemTime.wYear]));
        t:=False;
        end;

      'h','H':
        begin
        l:=Length(Result)+1;
        d:=Count('h')=1;
        if d then
          AddRes(SysUtils.Format('%d',[PersianSystemTime.wHour]))
        else
          AddRes(SysUtils.Format('%.2d',[PersianSystemTime.wHour]));
        t:=True;
        end;

      'n','N':
        begin
        if Count('n')=1 then
          AddRes(SysUtils.Format('%d',[PersianSystemTime.wMinute]))
        else
          AddRes(SysUtils.Format('%.2d',[PersianSystemTime.wMinute]));
        t:=False;
        end;

      's','S':
        begin
        if Count('s')=1 then
          AddRes(SysUtils.Format('%d',[PersianSystemTime.wSecond]))
        else
          AddRes(SysUtils.Format('%.2d',[PersianSystemTime.wSecond]));
        t:=False;
        end;

      'z','Z':
        begin
        if Count('z')<3 then
          AddRes(SysUtils.Format('%d',[PersianSystemTime.wMilliseconds]))
        else
          AddRes(SysUtils.Format('%.3d',[PersianSystemTime.wMilliseconds]));
        t:=False;
        end;

      't','T':
        begin
        if Count('t')=1 then
          Insert(FormatSettings.ShortTimeFormat,f,k)
        else
          Insert(FormatSettings.LongTimeFormat,f,k);
        t:=False;
        end;

      'a','A':
        begin
        n:=PersianSystemTime.wHour;
        if n=0 then n:=12;
        if n>12 then Dec(n,12);
        if IsChar(k+1,'m') and IsChar(k+2,'/') and IsChar(k+3,'p') and IsChar(k+4,'m') then
          begin
          if PersianSystemTime.wHour>=12 then
            AddRes(Copy(f,k+3,2))
          else
            AddRes(Copy(f,k,2));
          if l<>-1 then
            begin
            if d then
              begin
              Delete(Result,l,Length(SysUtils.Format('%d',[PersianSystemTime.wHour])));
              Insert(SysUtils.Format('%d',[n]),Result,l);
              end
            else
              begin
              Delete(Result,l,Length(SysUtils.Format('%.2d',[PersianSystemTime.wHour])));
              Insert(SysUtils.Format('%.2d',[n]),Result,l);
              end;
            l:=-1;
            end;
          Inc(k,5);
          end
        else if IsChar(k+1,'/') and IsChar(k+2,'p') then
          begin
          if PersianSystemTime.wHour>=12 then
            AddRes(Copy(f,k+2,1))
          else
            AddRes(Copy(f,k,1));
          if l<>-1 then
            begin
            if d then
              begin
              Delete(Result,l,Length(SysUtils.Format('%d',[PersianSystemTime.wHour])));
              Insert(SysUtils.Format('%d',[n]),Result,l);
              end
            else
              begin
              Delete(Result,l,Length(SysUtils.Format('%.2d',[PersianSystemTime.wHour])));
              Insert(SysUtils.Format('%.2d',[n]),Result,l);
              end;
            l:=-1;
            end;
          Inc(k,3);
          end
        else if IsChar(k+1,'m') and IsChar(k+2,'p') and IsChar(k+3,'m') then
          begin
          if PersianSystemTime.wHour>=12 then
            AddRes(FormatSettings.TimePMString)
          else
            AddRes(FormatSettings.TimeAMString);
          if l<>-1 then
            begin
            if d then
              begin
              Delete(Result,l,Length(SysUtils.Format('%d',[PersianSystemTime.wHour])));
              Insert(SysUtils.Format('%d',[n]),Result,l);
              end
            else
              begin
              Delete(Result,l,Length(SysUtils.Format('%.2d',[PersianSystemTime.wHour])));
              Insert(SysUtils.Format('%.2d',[n]),Result,l);
              end;
            l:=-1;
            end;
          Inc(k,4);
          end
        else
          begin
          AddRes(f[k]);
          Inc(k);
          end;
        end;

      '/':
        begin
        AddRes(FormatSettings.DateSeparator);
        Inc(k);
        end;

      ':':
        begin
        AddRes(FormatSettings.TimeSeparator);
        Inc(k);
        end;

    else
      begin
      AddRes(f[k]);
      Inc(k);
      end;
    end;
    end;
  end;
end;

procedure PersianSystemTimeToString(var Result:string;const Format:String;PersianSystemTime:TSystemTime);
begin
PersianSystemTimeToString(Result,Format,PersianSystemTime,PersianFormatSettings);
end;

procedure PersianTimeStampToString(var Result:string;const Format:String;PersianTimeStamp:TTimeStamp;const FormatSettings:TFormatSettings);
begin
PersianSystemTimeToString(Result,Format,PersianTimeStampToSystemTime(PersianTimeStamp),FormatSettings);
end;

procedure PersianTimeStampToString(var Result:string;const Format:String;PersianTimeStamp:TTimeStamp);
begin
PersianTimeStampToString(Result,Format,PersianTimeStamp,PersianFormatSettings);
end;

procedure PersianDateTimeToString(var Result:string;const Format:String;DateTime:TDateTime;const FormatSettings:TFormatSettings);
begin
PersianSystemTimeToString(Result,Format,DateTimeToPersianSystemTime(DateTime),FormatSettings);
end;

procedure PersianDateTimeToString(var Result:string;const Format:String;DateTime:TDateTime);
begin
PersianDateTimeToString(Result,Format,DateTime,PersianFormatSettings);
end;

function PersianDateToStr(const DateTime:TDateTime;const FormatSettings:TFormatSettings):string;
begin
PersianDateTimeToString(Result,FormatSettings.ShortDateFormat,DateTime,FormatSettings);
end;

function PersianDateToStr(const DateTime:TDateTime):string;
begin
Result:=PersianDateToStr(DateTime,PersianFormatSettings);
end;

function PersianDateTimeToStr(const DateTime:TDateTime;const FormatSettings:TFormatSettings):string;
begin
PersianDateTimeToString(Result,'',DateTime,FormatSettings);
end;

function PersianDateTimeToStr(const DateTime:TDateTime):string;
begin
Result:=PersianDateTimeToStr(DateTime,PersianFormatSettings);
end;

function FormatPersianDateTime(const Format:string;DateTime:TDateTime;const FormatSettings:TFormatSettings):string;
begin
PersianDateTimeToString(Result,Format,DateTime,FormatSettings);
end;

function FormatPersianDateTime(const Format:string;DateTime:TDateTime):string;
begin
Result:=FormatPersianDateTime(Format,DateTime,PersianFormatSettings);
end;

function EncodePersianDate(Year,Month,Day:Word):TDateTime;
begin
if not TryEncodePersianDate(Year,Month,Day,Result) then
  ConvertError(@SDateEncodeError);
end;

function TryEncodePersianDate(Year,Month,Day:Word;out Date:TDateTime):Boolean;
var
  s:TSystemTime;
begin
Result:=False;
if (Year=0) or (Month=0) or (Day=0) or (Month>12) or (Day>31) then Exit;
if (Month>6) and (Day>30) then Exit;
if (Month=12) and (Day=30) and (not IsPersianLeapYear(Year)) then Exit;
Result:=True;
FillChar(s,SizeOf(s),0);
s.wYear:=Year;
s.wMonth:=Month;
s.wDay:=Day;
Date:=PersianSystemTimeToDateTime(s);
end;

procedure DecodePersianDate(const DateTime:TDateTime;var Year,Month,Day:Word);
var d:Word;
begin
DecodePersianDateFully(DateTime,Year,Month,Day,d);
end;

function DecodePersianDateFully(const DateTime:TDateTime;var Year,Month,Day,DOW:Word):Boolean;
var s:TSystemTime;
begin
s:=DateTimeToPersianSystemTime(DateTime);
Year:=s.wYear;
Month:=s.wMonth;
Day:=s.wDay;
DOW:=s.wDayOfWeek+1;
Result:=IsPersianLeapYear(s.wYear);
end;

function ThePersianYear(aDate:TDateTime):Word;
begin
Result:=DateTimeToPersianSystemTime(aDate).wYear;
end;

function ThePersianMonth(aDate:TDateTime):Word;
begin
Result:=DateTimeToPersianSystemTime(aDate).wMonth;
end;

function ThePersianMonthShortName(aDate:TDateTime;const FormatSettings:TFormatSettings):String;
begin
Result:=FormatSettings.ShortMonthNames[ThePersianMonth(aDate)];
end;

function ThePersianMonthShortName(aDate:TDateTime):String;
begin
Result:=ThePersianMonthShortName(aDate,PersianFormatSettings);
end;

function ThePersianMonthLongName(aDate:TDateTime;const FormatSettings:TFormatSettings):String;
begin
Result:=FormatSettings.LongMonthNames[ThePersianMonth(aDate)];
end;

function ThePersianMonthLongName(aDate:TDateTime):String;
begin
Result:=ThePersianMonthLongName(aDate,PersianFormatSettings);
end;

function PersianDayOfTheMonth(aDate:TDateTime):Word;
begin
Result:=DateTimeToPersianSystemTime(aDate).wDay;
end;

function PersianDayOfTheYear(aDate:TDateTime):Word;
begin
Result:=Trunc(aDate-EncodePersianDate(ThePersianYear(aDate),1,1))+1;
end;

function PersianDayOfTheWeek(aDate:TDateTime):Word;
begin
Result:=DateTimeToPersianSystemTime(aDate).wDayOfWeek+1;
end;

function PersianDayOfTheWeekShortName(aDate:TDateTime;const FormatSettings:TFormatSettings):String;
begin
Result:=FormatSettings.ShortDayNames[PersianDayOfTheWeek(aDate)];
end;

function PersianDayOfTheWeekShortName(aDate:TDateTime):String;
begin
Result:=PersianDayOfTheWeekShortName(aDate,PersianFormatSettings);
end;

function PersianDayOfTheWeekLongName(aDate:TDateTime;const FormatSettings:TFormatSettings):String;
begin
Result:=FormatSettings.LongDayNames[PersianDayOfTheWeek(aDate)];
end;

function PersianDayOfTheWeekLongName(aDate:TDateTime):String;
begin
Result:=PersianDayOfTheWeekLongName(aDate,PersianFormatSettings);
end;

function CurrentPersianYear:Word;
begin
Result:=ThePersianYear(Date);
end;

function CurrentPersianMonth:Word;
begin
Result:=ThePersianMonth(Date);
end;

function CurrentPersianMonthShortName(const FormatSettings:TFormatSettings):String;
begin
Result:=ThePersianMonthShortName(Date,FormatSettings);
end;

function CurrentPersianMonthShortName:String;
begin
Result:=ThePersianMonthShortName(Date);
end;

function CurrentPersianMonthLongName(const FormatSettings:TFormatSettings):String;
begin
Result:=ThePersianMonthLongName(Date,FormatSettings);
end;

function CurrentPersianMonthLongName:String;
begin
Result:=ThePersianMonthLongName(Date);
end;

function CurrentPersianDayOfMonth:Word;
begin
Result:=PersianDayOfTheMonth(Date);
end;

function CurrentPersianDayOfYear:Word;
begin
Result:=PersianDayOfTheYear(Date);
end;

function CurrentPersianDayOfWeek:Word;
begin
Result:=PersianDayOfTheWeek(Date);
end;

function CurrentPersianDayOfWeekShortName(const FormatSettings:TFormatSettings):String;
begin
Result:=PersianDayOfTheWeekShortName(Date,FormatSettings);
end;

function CurrentPersianDayOfWeekShortName:String;
begin
Result:=PersianDayOfTheWeekShortName(Date);
end;

function CurrentPersianDayOfWeekLongName(const FormatSettings:TFormatSettings):String;
begin
Result:=PersianDayOfTheWeekLongName(Date,FormatSettings);
end;

function CurrentPersianDayOfWeekLongName:String;
begin
Result:=PersianDayOfTheWeekLongName(Date);
end;

function IncPersianMonth(const DateTime:TDateTime;NumberOfMonths:Integer=1):TDateTime;
var y,m,d:Word;
begin
DecodePersianDate(DateTime,y,m,d);
IncAPersianMonth(y,m,d,NumberOfMonths);
Result:=EncodePersianDate(y,m,d);
ReplaceTime(Result,DateTime);
end;

procedure IncAPersianMonth(var Year,Month,Day:Word;NumberOfMonths:Integer=1);
var
  d:PDayTable;
  s:Integer;
begin
if NumberOfMonths>=0 then s:=1 else s:=-1;
Year:=Year+(NumberOfMonths div 12);
NumberOfMonths:=NumberOfMonths mod 12;
Inc(Month,NumberOfMonths);
if Word(Month-1)>11 then    // if Month <= 0, word(Month-1) > 11)
  begin
  Inc(Year,s);
  Inc(Month,-12*s);
  end;
d:=@PersianMonthDays[IsPersianLeapYear(Year)];
if Day>d^[Month] then Day:=d^[Month];
end;

function DecPersianMonth(const DateTime:TDateTime;NumberOfMonths:Integer=1):TDateTime;
begin
Result:=IncPersianMonth(DateTime,-NumberOfMonths);
end;

procedure DecAPersianMonth(var Year,Month,Day:Word;NumberOfMonths:Integer=1);
begin
IncAPersianMonth(Year,Month,Day,NumberOfMonths-1);
end;

function IncPersianDay(const DateTime:TDateTime;NumberOfDays:Integer=1):TDateTime;
begin
Result:=DateTime+NumberOfDays;
end;

procedure IncAPersianDay(var Year,Month,Day:Word;NumberOfDays:Integer=1);
var d:TDateTime;
begin
d:=EncodePersianDate(Year,Month,Day);
IncPersianDay(d,NumberOfDays);
DecodePersianDate(d,Year,Month,Day);
end;

function DecPersianDay(const DateTime:TDateTime;NumberOfDays:Integer=1):TDateTime;
begin
Result:=IncPersianDay(DateTime,-NumberOfDays);
end;                    

procedure DecAPersianDay(var Year,Month,Day:Word;NumberOfDays:Integer=1);
begin
IncAPersianDay(Year,Month,Day,-NumberOfDays);
end;

function IncPersianWeek(const DateTime:TDateTime;NumberOfWeeks:Integer=1):TDateTime;
begin
Result:=IncPersianDay(DateTime,NumberOfWeeks*7);
end;

procedure IncAPersianWeek(var Year,Month,Day:Word;NumberOfWeeks:Integer=1);
begin
IncAPersianDay(Year,Month,Day,NumberOfWeeks*7);
end;

function DecPersianWeek(const DateTime:TDateTime;NumberOfWeeks:Integer=1):TDateTime;
begin
Result:=IncPersianWeek(DateTime,-NumberOfWeeks);
end;

procedure DecAPersianWeek(var Year,Month,Day:Word;NumberOfWeeks:Integer=1);
begin
IncAPersianWeek(Year,Month,Day,-NumberOfWeeks);
end;

function IncPersianYear(const DateTime:TDateTime;NumberOfYears:Integer=1):TDateTime;
var y,m,d:Word;
begin
DecodePersianDate(DateTime,y,m,d);
IncAPersianYear(y,m,d,NumberOfYears);
Result:=EncodePersianDate(y,m,d);
ReplaceTime(Result,DateTime);
end;

procedure IncAPersianYear(var Year,Month,Day:Word;NumberOfYears:Integer=1);
begin
Year:=Year+NumberOfYears;
if (Month=12) and (Day=30) and (not IsPersianLeapYear(Year)) then
  begin
  Month:=1;
  Day:=1;
  Inc(Year);
  end;
end;

function DecPersianYear(const DateTime:TDateTime;NumberOfYears:Integer=1):TDateTime;
begin
Result:=IncPersianYear(DateTime,-NumberOfYears);
end;

procedure DecAPersianYear(var Year,Month,Day:Word;NumberOfYears:Integer=1);
begin
DecAPersianYear(Year,Month,Day,-NumberOfYears);
end;

function StrToPersianDate(const S:string):TDateTime;
begin
if not TryStrToPersianDate(S,Result) then
  ConvertErrorFmt(@SInvalidDate,[S]);
end;

function StrToPersianDate(const S:string;const FormatSettings:TFormatSettings):TDateTime;
begin
if not TryStrToPersianDate(S,Result,FormatSettings) then
  ConvertErrorFmt(@SInvalidDate,[S]);
end;

function StrToPersianDateDef(const S:string;const Default:TDateTime):TDateTime;
begin
if not TryStrToPersianDate(S,Result) then Result:=Default;
end;

function StrToPersianDateDef(const S:string;const Default:TDateTime;const FormatSettings:TFormatSettings):TDateTime;
begin
if not TryStrToPersianDate(S,Result,FormatSettings) then Result:=Default;
end;

function TryStrToPersianDate(const S:string;out Value:TDateTime):Boolean;
begin
Result:=TryStrToPersianDate(S,Value,PersianFormatSettings);
end;

function TryStrToPersianDate(const S:string;out Value:TDateTime;const FormatSettings:TFormatSettings):Boolean;
var
  n1,n2,n3,t:String;
  y,m,d:Word;
  k:Integer;

  function GetDateOrder(const DateFormat:string):TDateOrder;
  var
    i:Integer;
  begin
  Result:=doMDY;
  i:=1;
  while i<=Length(DateFormat) do
    begin
    case Chr(Ord(DateFormat[I]) and $DF) of
      'E':Result:=doYMD;
      'Y':Result:=doYMD;
      'M':Result:=doMDY;
      'D':Result:=doDMY;
    else
      Inc(i);
      Continue;
    end;
    Exit;
    end;
  Result:=doMDY;
  end;

begin
y:=0;
m:=0;
d:=0;
Result:=False;
t:=S;
n3:='';
k:=Pos(FormatSettings.DateSeparator,t);
if (k<2) or (k=Length(t)) then Exit;
n1:=Copy(t,1,k-1);
Delete(t,1,k+Length(FormatSettings.DateSeparator)-1);
k:=Pos(FormatSettings.DateSeparator,t);
if k=1 then Exit;
if k=0 then
  n2:=t
else
  begin
  n2:=Copy(t,1,k-1);
  Delete(t,1,k+Length(FormatSettings.DateSeparator)-1);
  n3:=t;
  end;
for k:=1 to Length(n1) do
  if not {$if CompilerVersion > 19}CharInSet(n1[k],{$else}(n1[k] in{$ifend} ['0'..'9']) then
    Exit;
for k:=1 to Length(n2) do
  if not {$if CompilerVersion > 19}CharInSet(n2[k],{$else}(n2[k] in{$ifend} ['0'..'9']) then
    Exit;
for k:=1 to Length(n3) do
  if not {$if CompilerVersion > 19}CharInSet(n3[k],{$else}(n3[k] in{$ifend} ['0'..'9']) then
    Exit;
if (Length(n1)>4) or (Length(n2)>4) or (Length(n3)>4) then Exit;
case GetDateOrder(FormatSettings.ShortDateFormat) of
  doMDY:
    begin
    m:=StrToInt(n1);
    d:=StrToInt(n2);
    if n3<>'' then y:=StrToInt(n3);
    t:=n3;
    end;
  doDMY:
    begin
    d:=StrToInt(n1);
    m:=StrToInt(n2);
    if n3<>'' then y:=StrToInt(n3);
    t:=n3;
    end;
  doYMD:
    if n3='' then
      begin
      m:=StrToInt(n1);
      d:=StrToInt(n2);
      end
    else
      begin
      t:=n1;
      y:=StrToInt(n1);
      m:=StrToInt(n2);
      d:=StrToInt(n3);
      end;
end;
if n3='' then y:=CurrentPersianYear;
if (Length(t)<3) and (n3<>'') then y:=(CurrentPersianYear div 100)*100+y;
Result:=TryEncodePersianDate(y,m,d,Value);
end;

function StrToPersianDateTime(const S:string):TDateTime;
begin
if not TryStrToPersianDateTime(S,Result) then
  ConvertErrorFmt(@SInvalidDateTime,[S]);
end;

function StrToPersianDateTime(const S:string;const FormatSettings:TFormatSettings):TDateTime;
begin
if not TryStrToPersianDateTime(S,Result,FormatSettings) then
  ConvertErrorFmt(@SInvalidDateTime,[S]);
end;

function StrToPersianDateTimeDef(const S:string;const Default:TDateTime):TDateTime;
begin
if not TryStrToPersianDateTime(S,Result) then Result:=Default;
end;

function StrToPersianDateTimeDef(const S:string;const Default:TDateTime;const FormatSettings:TFormatSettings):TDateTime;
begin
if not TryStrToPersianDateTime(S,Result,FormatSettings) then Result:=Default;
end;

function TryStrToPersianDateTime(const S:string;out Value:TDateTime):Boolean;
begin
Result:=TryStrToPersianDateTime(S,Value,PersianFormatSettings);
end;

function TryStrToPersianDateTime(const S:string;out Value:TDateTime;const FormatSettings:TFormatSettings):Boolean;
var d,t:TDateTime;
begin
Result:=TryStrToPersianDate(S,d,FormatSettings) and TryStrToTime(S,t,FormatSettings);
if not Result then Exit;
if d<0 then
  Value:=d-t
else
  Value:=d+t;
end;

function ExtractPersianDateFromString(const S:string;DateOrder:TDateOrder;out Value:TDateTime):Boolean;
begin
Result:=ExtractPersianDateFromString(S,DateOrder,Value,PersianFormatSettings);
end;

function ExtractPersianDateFromString(const S:string;DateOrder:TDateOrder;out Value:TDateTime;const FormatSettings:TFormatSettings):Boolean;
var
  n1,n2,n3,t:String;
  y,m,d:Word;
  k:Integer;
  b:Boolean;

  function GetDateOrder(const DateFormat:string):TDateOrder;
  var
    i:Integer;
  begin
  Result:=doMDY;
  i:=1;
  while i<=Length(DateFormat) do
    begin
    case Chr(Ord(DateFormat[I]) and $DF) of
      'E':Result:=doYMD;
      'Y':Result:=doYMD;
      'M':Result:=doMDY;
      'D':Result:=doDMY;
    else
      Inc(i);
      Continue;
    end;
    Exit;
    end;
  Result:=doMDY;
  end;

begin
y:=0;
m:=0;
d:=0;
Result:=False;
t:=S;
n3:='';
while True do
  begin
  k:=Pos(FormatSettings.DateSeparator,t);
  if (k=0) or (k=Length(t)) then Break;
  if k=1 then
    begin
    Delete(t,1,Length(FormatSettings.DateSeparator));
    Continue;
    end;
  if (not {$if CompilerVersion > 19}CharInSet(t[k+1],{$else}(t[k+1] in{$ifend} ['0'..'9'])) or (not {$if CompilerVersion > 19}CharInSet(t[k-1],{$else}(t[k-1] in{$ifend} ['0'..'9'])) then
    begin
    Delete(t,1,k);
    Continue;
    end;
  n1:=Copy(t,1,k-1);
  Delete(t,1,k+Length(FormatSettings.DateSeparator)-1);
  k:=Pos(FormatSettings.DateSeparator,t);
  if k=1 then Break;
  if k=0 then
    n2:=t
  else
    begin
    n2:=Copy(t,1,k-1);
    Delete(t,1,k+Length(FormatSettings.DateSeparator)-1);
    n3:=t;
    b:=False;
    for k:=1 to Length(n2) do
      if not {$if CompilerVersion > 19}CharInSet(n2[k],{$else}(n2[k] in{$ifend} ['0'..'9']) then
        begin
        b:=True;
        Break;
        end;
    if b then Break;
    end;
  k:=1;
  while k<=Length(n1) do
    if not {$if CompilerVersion > 19}CharInSet(n1[k],{$else}(n1[k] in{$ifend} ['0'..'9']) then
      begin
      Delete(n1,1,k);
      k:=1;
      end
    else
      Inc(k);
  k:=Length(n3);
  while k>0 do
    if not {$if CompilerVersion > 19}CharInSet(n3[k],{$else}(n3[k] in{$ifend} ['0'..'9']) then
      begin
      Delete(n3,k,Length(n3)-k+1);
      k:=Length(n3);;
      end
    else
      Dec(k);
  if (Length(n1)>4) or (Length(n2)>4) or (Length(n3)>4) then Break;
  case DateOrder of
    doMDY:
      begin
      m:=StrToInt(n1);
      d:=StrToInt(n2);
      if n3<>'' then y:=StrToInt(n3);
      t:=n3;
      end;
    doMYD:
      begin
      m:=StrToInt(n1);
      if n3<>'' then
        begin
        t:=n2;
        y:=StrToInt(n2);
        d:=StrToInt(n3);
        end
      else
        d:=StrToInt(n2);
      end;
    doDMY:
      begin
      d:=StrToInt(n1);
      m:=StrToInt(n2);
      if n3<>'' then y:=StrToInt(n3);
      t:=n3;
      end;
    doDYM:
      begin
      d:=StrToInt(n1);
      if n3<>'' then
        begin
        t:=n2;
        y:=StrToInt(n2);
        m:=StrToInt(n3);
        end
      else
        m:=StrToInt(n2);
      end;
    doYMD:
      if n3='' then
        begin
        m:=StrToInt(n1);
        d:=StrToInt(n2);
        end
      else
        begin
        t:=n1;
        y:=StrToInt(n1);
        m:=StrToInt(n2);
        d:=StrToInt(n3);
        end;
    doYDM:
      if n3='' then
        begin
        d:=StrToInt(n1);
        m:=StrToInt(n2);
        end
      else
        begin
        t:=n1;
        y:=StrToInt(n1);
        d:=StrToInt(n2);
        m:=StrToInt(n3);
        end;

  end;
  if n3='' then y:=CurrentPersianYear;
  if (Length(t)<3) and (n3<>'') then y:=(CurrentPersianYear div 100)*100+y;
  Result:=TryEncodePersianDate(y,m,d,Value);
  Break;
  end;
end;

function NumberToPersianText(I:Int64):String;
const
  y:array[0..19] of string = ('','','','','','','','','','','','','','','','','','','','');
  d:array[2..9] of string = ('','','','','','','','');
  s:array[0..9] of string = ('','ߝ','','흝','','','ԝ','ʝ','ʝ','坝');

var
 o:string;

  function dd(n:Int64):String;
  var p,t:string;
  begin
  Result:=s[n div 100];
  n:=n mod 100;
  if n>=20 then
    begin
    p:=d[n div 10];
    n:=n mod 10;
    t:=y[n];
    if t<>'' then p:=p+'  '+t;
    end
  else
    p:=y[n mod 20];
  if (p<>'') and (Result<>'') then Result:=Result+'  '+p;
  if Result='' then Result:=p;
  end;

begin
if I<0 then I:=-I;
o:='';
if I=0 then
  Result:=''
else if I<1000 then
  Result:=dd(I)
else if I<1000000 then
  begin
  Result:=dd(I div 1000);
  o:=dd(I mod 1000);
  if o<>'' then Result:=Result+'   '+o else Result:=Result+' ';
  end
else if I<1000000000 then
  begin
  Result:=dd(I div 1000000);
  if I mod 1000000<0 then o:=NumberToPersianText(I mod 1000000);
  if o<>'' then Result:=Result+'   '+o else Result:=Result+' ';
  end
else if I<1000000000000 then
  begin
  Result:=NumberToPersianText(I div 1000000000);
  if I mod 1000000000<>0 then o:=NumberToPersianText(I mod 1000000000);
  if o<>'' then Result:=Result+'   '+o else Result:=Result+' ';
  end
else
  begin
  Result:=NumberToPersianText(I div 1000000000000);
  if I mod 1000000000000<>0 then o:=NumberToPersianText(I mod 1000000000000);
  if o<>'' then Result:=Result+'   '+o else Result:=Result+' ';
  end;
end;

function NumberToPersianOrdinalText(I:Int64):String;
begin
if I<0 then I:=-I;
Result:=NumberToPersianText(I);
if (I mod 10=3) and (I mod 100<>13) then
  begin
  Delete(Result,Length(Result),1);
  Result:=Result+'';
  end;
if I mod 100=30 then
  begin
  Result:=Result+' ';
  end;
Result:=Result+'';
if I=1 then Result:='';
if I=0 then Result:='';
end;

function NumberToOrdinalNumber(I:Int64):String;
begin
if I<0 then I:=-I;
Result:=IntToStr(I);
if I=0 then Exit;
if (I mod 10=1) and (I mod 100<>11) then
  Result:=Result+'st'
else if (I mod 10=2) and (I mod 100<>12)  then
  Result:=Result+'nd'
else if (I mod 10=3) and (I mod 100<>13)  then
  Result:=Result+'rd'
else
  Result:=Result+'th';
end;

function NumberToText(I:Int64):String;
const
  y:array[0..19] of string = ('','one','two','three','four','five','six','seven','eight','nine','ten','eleven','twelve','thirteen','fourteen','fifteen','sixteen','seventeen','eighteen','nineteen');
  d:array[2..9] of string = ('twenty','thirty','forty','fifty','sixty','seventy','eighty','ninety');
  s:array[0..9] of string = ('','one hundred','two hundred','three hundred','four hundred','five hundred','six hundred','seven hundred','eight hundred','nine hundred');

var
 o:string;

  function dd(n:Int64):String;
  var p,t:string;
  begin
  Result:=s[n div 100];
  n:=n mod 100;
  if n>=20 then
    begin
    p:=d[n div 10];
    n:=n mod 10;
    t:=y[n];
    if t<>'' then p:=p+' '+t;
    end
  else
    p:=y[n mod 20];
  if (p<>'') and (Result<>'') then Result:=Result+' '+p;
  if Result='' then Result:=p;
  end;

begin
if I<0 then I:=-I;
o:='';
if I=0 then
  Result:='zero'
else if I<1000 then
  Result:=dd(I)
else if I<1000000 then
  begin
  Result:=dd(I div 1000);
  o:=dd(I mod 1000);
  if o<>'' then Result:=Result+' thousand '+o else Result:=Result+' thousand';
  end
else if I<1000000000 then
  begin
  Result:=dd(I div 1000000);
  if I mod 1000000<>0 then o:=NumberToText(I mod 1000000);
  if o<>'' then Result:=Result+' million '+o else Result:=Result+' million';
  end
else if I<1000000000000 then
  begin
  Result:=NumberToText(I div 1000000000);
  if I mod 1000000000<>0 then o:=NumberToText(I mod 1000000000);
  if o<>'' then Result:=Result+' billion '+o else Result:=Result+' billion';
  end
else
  begin
  Result:=NumberToText(I div 1000000000000);
  if I mod 1000000000000<>0 then o:=NumberToText(I mod 1000000000000);
  if o<>'' then Result:=Result+' trillion '+o else Result:=Result+' trillion';
  end
end;

function NumberToOrdinalText(I:Int64):String;
const
  y:array[0..19] of string = ('','first','second','third','fourth','fifth','sixth','seventh','eighth','ninth','tenth','eleventh','twelfth','thirteenth','fourteenth','fifteenth','sixteenth','seventeenth','eighteenth','nineteenth');
  d:array[2..9] of string = ('twenty','thirty','forty','fifty','sixty','seventy','eighty','ninety');
  s:array[0..9] of string = ('','one hundred','two hundred','three hundred','four hundred','five hundred','six hundred','seven hundred','eight hundred','nine hundred');

var
 o:string;

  function dd(n:Int64):String;
  var p,t:string;
  begin
  Result:=s[n div 100];
  n:=n mod 100;
  if n>=20 then
    begin
    p:=d[n div 10];
    n:=n mod 10;
    t:=y[n];
    if t<>'' then p:=p+' '+t;
    end
  else
    p:=y[n mod 20];
  if (p<>'') and (Result<>'') then Result:=Result+' '+p;
  if Result='' then Result:=p;
  end;

begin
o:='';
if I<0 then I:=-I;
if I=0 then
  Result:='zero'
else if I<1000 then
  Result:=dd(I)
else
  begin
  Result:=NumberToText((I div 1000)*1000);
  if I mod 1000<>0 then Result:=Result+' '+dd(I mod 1000);
  end;
if (I mod 10=0) and (I mod 100<>10) and (I<>0) then
  begin
  if Result[Length(Result)]='y' then
    begin
    Result[Length(Result)]:='i';
    Result:=Result+'e';
    end;
  Result:=Result+'th';
  end;
end;

end.
